;; Generated package description from nadvice.el  -*- no-byte-compile: t -*-
(define-package "nadvice" "0.4.0.20230111.104526" "Forward compatibility for Emacs-24.4's nadvice" 'nil :commit "d19e9ae649374a0f2fab1baa045ed192e0851750" :url "https://elpa.gnu.org/packages/nadvice.html" :authors '(("Stefan Monnier" . "monnier@iro.umontreal.ca")) :maintainer '("Stefan Monnier" . "monnier@iro.umontreal.ca"))
